<?php


namespace App\Mobile\Synchronization\Managers;


use Schaffen\Core\Person\Managers\AddressManager;
use Schaffen\Core\Person\Managers\PersonManager;
use Schaffen\Core\Person\Repositories\PersonRepository;
use Schaffen\Core\Product\Repositories\ProductVarietyPackRepository;
use Schaffen\Core\User\Models\User;
use Schaffen\Core\Utils\Services\RecursiveCollectConvert;
use Schaffen\Core\Visitation\Managers\VisitationManager;
use Schaffen\Financial\FinancialMovement\Models\ImportedFinancialMovement;
use Schaffen\Financial\FinancialMovement\Repositories\ImportedFinancialMovementRepository;
use Schaffen\HumanResources\HumanResources\Managers\ClockManager;
use Schaffen\ServiceOrder\ServiceOrder\Managers\ServiceOrderManager;
use Schaffen\ServiceOrder\ServiceOrder\Models\ServiceOrder;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderRepository;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderResourceRepository;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceOrderStatusRepository;
use Schaffen\ServiceOrder\ServiceOrder\Repositories\ServiceTypeRepository;

class SyncManager
{

    /**
     * @var PersonManager
     */
    private PersonManager $person_manager;
    /**
     * @var AddressManager
     */
    private AddressManager $address_manager;
    /**
     * @var VisitationManager
     */
    private VisitationManager $visitation_manager;
    /**
     * @var ServiceOrderManager
     */
    private ServiceOrderManager $service_order_manager;
    /**
     * @var ClockManager
     */
    private ClockManager $clock_manager;
    /**
     * @var ServiceTypeRepository
     */
    private ServiceTypeRepository $service_type_repository;
    /**
     * @var ServiceOrderStatusRepository
     */
    private ServiceOrderStatusRepository $service_order_status_repository;
    /**
     * @var ServiceOrderRepository
     */
    private ServiceOrderRepository $service_order_repository;
    private ProductVarietyPackRepository $productVarietyPackRepository;
    private ImportedFinancialMovementRepository $importedFinancialMovementRepository;
    private ServiceOrderResourceRepository $service_order_resource_repository;

    public function __construct(PersonManager $person_manager, AddressManager $address_manager,
                                VisitationManager $visitation_manager, ServiceOrderManager $service_order_manager,
                                ClockManager $clock_manager, ServiceTypeRepository $service_type_repository,
                                ServiceOrderStatusRepository $service_order_status_repository,
                                ServiceOrderRepository $service_order_repository,
                                ProductVarietyPackRepository $productVarietyPackRepository,
                                ImportedFinancialMovementRepository $importedFinancialMovementRepository,
                                ServiceOrderResourceRepository $service_order_resource_repository
    )
    {
        $this->service_order_repository = $service_order_repository;
        $this->person_manager = $person_manager;
        $this->address_manager = $address_manager;
        $this->visitation_manager = $visitation_manager;
        $this->service_order_manager = $service_order_manager;
        $this->clock_manager = $clock_manager;
        $this->service_type_repository = $service_type_repository;
        $this->service_order_status_repository = $service_order_status_repository;
        $this->productVarietyPackRepository = $productVarietyPackRepository;
        $this->importedFinancialMovementRepository = $importedFinancialMovementRepository;
        $this->service_order_resource_repository = $service_order_resource_repository;
    }


    public function uploadSync(array $values)
    {
        $clients = [];
        $addresses = [];
        $service_orders = [];
        $clocks = [];

        extract($values);

        foreach ($clients as $client) {
            $client['pes_cad_incompleto'] = true;
            $this->person_manager->storeOrUpdate($client);
        }
        foreach ($addresses as $address) {
            $this->address_manager->storeOrUpdate($address, $address['pee_codigo']);
        }

        /** @var User $user */
        $user = auth()->user();
        $usu_codigo = $user->usu_codigo;
        $pes_codigo = $user->usu_fk_pes_codigo;

        /** @var int $service_type_id */
        $service_type_id = optional($this->service_type_repository
            ->setSelect('tse_codigo')
            ->first(['tse_descricao' => 'OS']))->tse_codigo;

        /** @var int $service_status_id */
        $service_status_id = $this->service_order_status_repository->getStatusExecutado(ServiceOrder::SERVICE_ORDER) ?? 1;

        /** @var int $last_os_numero_tipo */
        $last_os_numero_tipo = $this->service_order_repository->last_os_numero_tipo(ServiceOrder::SERVICE_ORDER);

        foreach ($service_orders as $service_order) {
            $os = optional($service_order);
            if (
                (!empty($service_order["os_data_agendamento"]) && $service_order["os_data_agendamento"] >= now()->endOfDay()) &&
                ($service_order["os_fk_sto_codigo"] != ServiceOrder::STATUS_EM_EXECUCAO)
            ) continue;

            if ($os->offsetExists('visitation')) {
                if (is_string($service_order['visitation']['vis_distancia'] ?? NULL)) {
                    $service_order['os_end_distancia_km'] = $service_order['visitation']['vis_distancia'] ?? 0;
                    $service_order['visitation']['vis_distancia'] = (int)str_replace('.', '', $service_order['visitation']['vis_distancia']);
                }
            }

//            $visitation = false;
//            if ($service_order['os_planejada'] == false) {
//                $visitation = true;
//            }

            $service_order['items'] = collect($service_order['items'])
                ->map(function ($item) use ($pes_codigo) {
                    if (isset($item['osps_fk_pve_codigo'])) {
                        $product = $this->productVarietyPackRepository->find($item['osps_fk_pve_codigo'], ['product_variety']);
                        $item['osps_valor_unitario'] = $product->product_variety->prv_valor_venda;
                    }
                    if (!isset($item['osps_codigo'])) {
                        $item['osps_recurso'] = ['pes_codigo' => $pes_codigo];
                    }
                    return $item;
                })->toArray();
            if (!isset($service_order['os_fk_sto_codigo']) || empty($service_order['os_fk_sto_codigo'])) {
                $service_order['os_fk_sto_codigo'] = NULL;
            }

            if (!($input['os_fk_sto_codigo'] ?? NULL)) {
                $service_order_status_repository = app()->make(ServiceOrderStatusRepository::class);
                $input['os_fk_sto_codigo'] = $service_order_status_repository->first(["sto_status_inicial" => 1])->sto_codigo;
            }

            if (!isset($service_order['os_data_agendamento'])) {
                $service_order['os_data_agendamento'] = $service_order['os_data_fechamento'] ?? NULL;
            }

            if (
                !isset($service_order['os_fk_sto_codigo']) &&
                (!isset($service_order['os_concluida']) ||
                    (is_bool($service_order['os_concluida']) ? $service_order['os_concluida'] : $service_order['os_concluida'] == 'false') &&
                    isset($service_order['os_codigo']))
            ) {
                $service_order_repository = app()->make(ServiceOrderRepository::class);
                $service_order['os_fk_sto_codigo'] = $service_order_repository->first(
                    [
                        "os_codigo" => $service_order['os_codigo']
                    ])->sto_codigo;

            }

            //TODO Adicionar status padrão para a OS
            //TODO Discriminação do serviço
            $service_order['os_fk_tse_codigo'] = $service_type_id; //Tipo de serviço como OS
            $service_order['os_fk_sto_codigo'] = isset($service_order['os_concluida']) && (bool)$service_order['os_concluida'] ? $service_status_id : $service_order['os_fk_sto_codigo']; //Tipo de serviço como OS
            $service_order['os_tipo'] = ServiceOrder::SERVICE_ORDER;
            $service_order['os_fk_usu_codigo_vendedor'] = $usu_codigo;
            if (!isset($service_order['os_codigo'])) {
                $person_repo = app()->make(PersonRepository::class);
                $client = $person_repo->find($service_order['os_fk_pes_codigo']);
                if (isset($client) && $client->pes_fiscal_tipo_cliente == 1) {
                    $comissao_produto = optional(optional($user->person)->Employee)->fun_comissao_venda ?? 0;
                    $comissao_servico = optional(optional($user->person)->Employee)->fun_comissao_servico ?? 0;
                } else {
                    $comissao_produto = optional(optional($user->person)->Employee)->fun_comissao_venda_final ?? 0;
                    $comissao_servico = optional(optional($user->person)->Employee)->fun_comissao_servico_final ?? 0;
                }

                $resource = $person_repo->find($pes_codigo);

                $service_order['os_recursos'] = [
                    [
                        "osr_fk_vendedor_codigo" => $pes_codigo,
                        "osr_percentual_comissao_produto" => $comissao_produto,
                        "osr_percentual_comissao_servico" => $comissao_servico,
                        "recurso" => [
                            "pes_codigo" => $pes_codigo,
                            "pes_nome" => $resource->pes_nome
                        ]
                    ]
                ];
            }


            $service_order['os_campos'] = [];

            $service_order['os_numero_tipo'] = ++$last_os_numero_tipo;
            if ($os->offsetExists('visitation')) {
                $visitation = $service_order['visitation'];
                $service_order['os_observacao'] = $visitation['vis_observacao'] ?? NULL;
            } else {
                $service_order['os_observacao'] = NULL;
            }

            $service_order['os_end_distancia_km'] = $service_order['os_end_distancia_km'] ?? 0;
            $os_codigo = $this->service_order_manager->storeOrUpdate($service_order, isset($service_order['os_codigo']) ? $service_order['os_codigo'] : NULL)->os_codigo;

            //Store Visit
            if ($os->offsetExists('visitation')) {
                $sanitized = array_merge([
                    'vis_visitavel_codigo' => $os_codigo,
                    'vis_visitavel_tipo' => 'service-order',
                    'vis_fk_pes_codigo_vdr' => $pes_codigo
                ], $visitation);

                $visit = $this->visitation_manager->hasVisit($sanitized);

                if (is_null($visit)) {
                    $this->visitation_manager->storeOrUpdate($sanitized);
                }

            }
        }

        foreach ($clocks as $clock) {
            $clock['pnt_fk_usu_codigo'] = $user->usu_codigo;
            $this->clock_manager->storeOrUpdate($clock);
        }
    }

    public function uploadSyncRoute($routeValues)
    {
        $mvfs = [];
        $unexpected_payments = [];
        extract($routeValues);

        $mvfsCollection = RecursiveCollectConvert::r_collection($mvfs);
        $mvfsCollection->each(function ($mvf) {
            $mvf->parcelas->each(function ($parcela) {
                $this->importedFinancialMovementRepository->update([
                    'mfim_fk_par_codigo' => $parcela->par_codigo
                ], [
                    'mfim_valor' => $parcela->valor,
                    'mfi_data_importado' => now(),
                    'mfim_descricao' => $parcela->descricao,
                    'mfim_status' => ImportedFinancialMovement::STATUS_IMPORTADO
                ]);
            });
        });

        //TODO:verificar regras para pagamentos não esperados
        if (!empty($unexpected_payments) && is_array($unexpected_payments)) {
            $unexpectedPaymentsCollection = RecursiveCollectConvert::r_collection($unexpected_payments);
            $unexpectedPaymentsCollection->each(function ($payment) {
                $description = '';
                if (isset($payment->pes_codigo)) {
                    $description .= ' | pes_codigo = ' . $payment->pes_codigo;
                }
                if (isset($payment->pes_name)) {
                    $description .= ' | pes_codigo = ' . $payment->pes_name;
                }
                if ($description == '') {
                    $description = ' | pagamento não identificado';
                }
                $this->importedFinancialMovementRepository->storeOrUpdate([
                    'mfim_valor' => $payment->valor,
                    'mfi_data_importado' => now(),
                    'mfim_descricao' => $payment->descricao . $description,
                    'mfim_status' => ImportedFinancialMovement::STATUS_IMPORTADO
                ]);
            });
        }
    }
}
